<?php


namespace Mediavine\Create\Settings;

use Mediavine\Create\Plugin;

class Control_Panel implements Settings_Group {

	/**
	 * Check for the existence of MV Control Panel before adding settings
	 * @return bool
	 */
	private static function has_mvcp() {
		return ( class_exists( 'MV_Control_Panel' ) || class_exists( 'MVCP' ) );
	}

	/**
	 * @inheritDoc
	 */
	public static function settings() {
		if ( ! self::has_mvcp() ) {
			return [];
		}

		return [
			[
				'slug'  => Plugin::$settings_group . '_ad_density',
				'value' => '583',
				'group' => Plugin::$settings_group . '_mvp',
				'order' => 100,
				'data'  => [
					'type'         => 'select',
					'label'        => __( 'Ad Density', 'mediavine' ),
					'instructions' => __( 'Choose the density of ads in a recipe card.', 'mediavine' ),
					'default'      => __( 'Normal', 'mediavine' ),
					'options'      => [
						[
							'label' => __( 'Single Ad', 'mediavine' ),
							'value' => '0',
						],
						[
							'label' => __( 'Normal', 'mediavine' ),
							'value' => '583',
						],
					],
				],
			],
			[
				'slug'  => Plugin::$settings_group . '_list_items_between_ads',
				'value' => '3',
				'group' => Plugin::$settings_group . '_mvp',
				'order' => 110,
				'data'  => [
					'type'         => 'select',
					'label'        => __( 'List Items Between Ads', 'mediavine' ),
					'instructions' => __( 'Choose the number of list items between each ad in the card.', 'mediavine' ),
					'options'      => [
						[
							'label' => __( 'Disable ads in lists', 'mediavine' ),
							'value' => 0,
						],
						[
							'label' => __( '2', 'mediavine' ),
							'value' => '2',
						],
						[
							'label' => __( '3', 'mediavine' ),
							'value' => '3',
						],
						[
							'label' => __( '4', 'mediavine' ),
							'value' => '4',
						],
						[
							'label' => __( '5', 'mediavine' ),
							'value' => '5',
						],
					],
				],
			],
		];
	}
}
