<?php

namespace Mediavine\Create;

use Mediavine\Create\Monolog\Logger;
use Mediavine\Create\Monolog\Handler\AbstractProcessingHandler;
class Raven_Breadcrumbs_MonologHandler extends \Mediavine\Create\Monolog\Handler\AbstractProcessingHandler
{
    /**
     * Translates Monolog log levels to Raven log levels.
     */
    protected $logLevels = array(\Mediavine\Create\Monolog\Logger::DEBUG => \Mediavine\Create\Raven_Client::DEBUG, \Mediavine\Create\Monolog\Logger::INFO => \Mediavine\Create\Raven_Client::INFO, \Mediavine\Create\Monolog\Logger::NOTICE => \Mediavine\Create\Raven_Client::INFO, \Mediavine\Create\Monolog\Logger::WARNING => \Mediavine\Create\Raven_Client::WARNING, \Mediavine\Create\Monolog\Logger::ERROR => \Mediavine\Create\Raven_Client::ERROR, \Mediavine\Create\Monolog\Logger::CRITICAL => \Mediavine\Create\Raven_Client::FATAL, \Mediavine\Create\Monolog\Logger::ALERT => \Mediavine\Create\Raven_Client::FATAL, \Mediavine\Create\Monolog\Logger::EMERGENCY => \Mediavine\Create\Raven_Client::FATAL);
    protected $excMatch = '/^exception \'([^\']+)\' with message \'(.+)\' in .+$/s';
    /**
     * @var Raven_Client the client object that sends the message to the server
     */
    protected $ravenClient;
    /**
     * @param Raven_Client $ravenClient
     * @param int          $level       The minimum logging level at which this handler will be triggered
     * @param bool         $bubble      Whether the messages that are handled can bubble up the stack or not
     */
    public function __construct(\Mediavine\Create\Raven_Client $ravenClient, $level = \Mediavine\Create\Monolog\Logger::DEBUG, $bubble = \true)
    {
        parent::__construct($level, $bubble);
        $this->ravenClient = $ravenClient;
    }
    /**
     * @param string $message
     * @return array|null
     */
    protected function parseException($message)
    {
        if (\preg_match($this->excMatch, $message, $matches)) {
            return array($matches[1], $matches[2]);
        }
        return null;
    }
    /**
     * {@inheritdoc}
     */
    protected function write(array $record)
    {
        // sentry uses the 'nobreadcrumb' attribute to skip reporting
        if (!empty($record['context']['nobreadcrumb'])) {
            return;
        }
        if (isset($record['context']['exception']) && ($record['context']['exception'] instanceof \Exception || \PHP_VERSION_ID >= 70000 && $record['context']['exception'] instanceof \Throwable)) {
            /**
             * @var \Exception|\Throwable $exc
             */
            $exc = $record['context']['exception'];
            $crumb = array('type' => 'error', 'level' => $this->logLevels[$record['level']], 'category' => $record['channel'], 'data' => array('type' => \get_class($exc), 'value' => $exc->getMessage()));
        } else {
            // TODO(dcramer): parse exceptions out of messages and format as above
            if ($error = $this->parseException($record['message'])) {
                $crumb = array('type' => 'error', 'level' => $this->logLevels[$record['level']], 'category' => $record['channel'], 'data' => array('type' => $error[0], 'value' => $error[1]));
            } else {
                $crumb = array('level' => $this->logLevels[$record['level']], 'category' => $record['channel'], 'message' => $record['message'], 'data' => $record['context']);
            }
        }
        $this->ravenClient->breadcrumbs->record($crumb);
    }
}
\class_alias('Mediavine\\Create\\Raven_Breadcrumbs_MonologHandler', 'Raven_Breadcrumbs_MonologHandler', \false);
